// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.
#version 330 core

uniform sampler2D texture2; // arrow texture
uniform sampler1D texture4; // colormap
uniform float alpha0;
uniform vec2 range;
uniform float logscale;

in vec2 texCoord0;
flat in float angle;
flat in float windSpeedKnot;

out vec4 fragColor;

// Clamp the value between [min, max] and normalize to [0, 1]
float saturateToRange(in float value, in vec2 range)
{
    float result = clamp(value, range.x, range.y);
    return (result - range.x) / (range.y - range.x);
}

void main() 
{
    float alpha = texture(texture2, texCoord0).r;
    float value = windSpeedKnot;
    if (logscale > 0.0)
    {
        value = (log(value) - log(range.x)) / (log(range.y) - log(range.x));
    }
    else
    {
        value = saturateToRange(value, range);
    }
    vec3 color = texture(texture4, value).rgb;

    fragColor = vec4(color, min(alpha, alpha0));
}
