// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330 core

const float PI = 3.141592653589793;

uniform sampler1D texture2; // colormap
uniform float alpha;

in float lineWidth;
in vec2 texCoord0;
flat in float segmentLength;
in float data0;

out vec4 color1;

void main()
{
    float dx = texCoord0.x;
    float dy = abs(texCoord0.y);
    float d = dy;
    
    float antialias = 1.0;
    //float t = (thickness / halfViewport.x / 2.0);// - (1.5/400.0)*antialias;
    float t = lineWidth;// - (1.5/400.0)*antialias;

    if (texCoord0.x < 0.0)
    {
        d = length(texCoord0);
    }
    else if (texCoord0.x > segmentLength)
    {
        d = length(vec2(texCoord0.x - segmentLength, texCoord0.y));
    }

    d = d - t;

    float a = 1.0;

    if(d > 0.0)
    {
        //d /= antialias;
        //float a = exp(-(d*halfViewport.x)*(d*halfViewport.x));// * color.a;
        a = exp(-d*d);// * color.a;
        if( a < 0.001 )
            discard;
    }

    color1 = vec4(texture(texture2, data0).rgb, alpha * a);
}
