// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330 core

uniform sampler2D texture0; // data
uniform sampler2D texture1; // mask

uniform vec3 scaleFactor;
uniform vec3 offset;
uniform float multiplicator;

#define readTexture3(texUnit, coord) ((texture(texUnit, coord).rgb * multiplicator) * scaleFactor) + offset

in vec2 texCoord0;
out vec4 out_Color;

void main()
{
    float mask = texture(texture1, texCoord0).r;

    if (mask > 0.0)
    {
        vec3 value = readTexture3(texture0, texCoord0);
        out_Color = vec4(value.bgr, 1.0);
    }
    else
    {
        discard;
    }
}
