// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330 core

layout(location = 0) in float percent;

uniform vec3 p0;
uniform vec3 p1;
uniform mat4 mvp;
uniform vec2 angles;

vec3 slerp(vec3 p0, vec3 p1, float t)
{
    float dotp = dot(normalize(p0), normalize(p1));

    if ((dotp > 0.9999) || (dotp<-0.9999))
    {
        if (t<=0.5)
            return p0;
        return p1;
    }

    float theta = acos(dotp * 3.14159/180.0);
    return ((p0*sin((1.0-t)*theta) + p1*sin(t*theta)) / sin(theta));
}

vec3 ToVector3D(vec2 geodetic)
{
    float cosLatitude = cos(geodetic.y);

    return normalize(vec3(
        cosLatitude * cos(geodetic.x),
        cosLatitude * sin(geodetic.x),
        sin(geodetic.y)));
}

void main()
{
    vec3 p = normalize(slerp(p0, p1, percent));
    gl_Position = mvp * vec4(p, 1.0);
}
