// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330

in vec2 position;

uniform vec4 top;
uniform vec4 bottom;
uniform mat4 mvp;

#define PI      3.14159265359
#define TWO_PI  6.28318530718

vec3 ToVector3D(vec2 geodetic)
{
    float cosLatitude = cos(geodetic.y);

    return normalize(vec3(
        cosLatitude * cos(geodetic.x),
        cosLatitude * sin(geodetic.x),
        sin(geodetic.y)));
}

float recenter(float a)
{
    float b;

    if (a > PI)
    {
        b = a - TWO_PI;
    }
    else
    {
        b = a;
    }
    return b;
}

void main()
{
    vec2 topMix = mix(top.xy, top.zw, position.x);
    vec2 bottomMix = mix(bottom.xy, bottom.zw, position.x);
    vec2 lonlat = mix(topMix, bottomMix, position.y);

    //lonlat.x = recenter(lonlat.x);

    gl_Position = mvp * vec4(ToVector3D(lonlat), 1.0);
}
