// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330 core

layout(location = 0) in vec3 position;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 1) in float localID;
//layout(location = 2) in float time;


uniform sampler2D texture0; // gcp

uniform mat4 mvp;
uniform float streamlineLength;
uniform vec3 fillColor;
uniform vec2 size;
uniform vec2 texelSize;
uniform vec2 uvMin;
uniform vec2 uvRange;
uniform float ttl;
uniform float fadeIn;
uniform float fadeOut;
uniform float currentTime;

out vec4 color0;

void main()
{
    float alpha = 1.0;
    float time = position.z;

    if (currentTime - time < fadeIn)
    {
        alpha = (clamp(currentTime - time, 0.0, fadeIn) / fadeIn); // [0, fade] -> [0, 1]
    }
    else if ((currentTime - time) > ttl + fadeIn)
    {
        alpha = 1.0 - (clamp(currentTime - time - ttl - fadeIn, 0.0, fadeOut) / fadeOut); // [0, fade] -> [0, 1]
    }

    color0.a = min(alpha, localID / streamlineLength);
    color0.rgb = fillColor;

    vec2 particlePos = (position.xy - uvMin) / uvRange;
    vec3 particlePos3 = normalize(texture(texture0, position.xy).xyz);

    gl_Position = mvp * vec4(particlePos3, 1.0);
}
