// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330 core

layout(location = 0) in vec2 direction;
layout(location = 1) in vec2 texCoord;

uniform mat4 mvp;
uniform vec2 viewport;
// half diagonal size, square edge = (arrowTileSize * 2) / sqrt(2)
uniform float arrowTileSize; 
uniform vec3 position;

out vec2 texCoord0;

vec2 project(vec4 P)
{
    vec2 p = 0.5 + (P.xyz/P.w).xy * 0.5;
    return p * viewport;
}

vec4 unproject(vec2 p, float z, float w)
{
    vec4 P = vec4( w*((p/viewport)*2.0 - 1.0), z, w);
    return P;
}

void main() 
{
    texCoord0 = texCoord;

    vec3 pos = position;
    
    vec4 currentProjected = mvp * vec4(pos, 1.0);
    vec2 currentScreen    = project(currentProjected);

    // Compute pixel coordinates in Normalized Device Coordinates space [0, 1].
    vec2 pixelCoord = 0.5 + (currentProjected.xyz/currentProjected.w).xy * 0.5;

    // Compute the position of the current corner vertex in screen space
    vec2 dirNorm = normalize(direction.xy);

    vec2 currentScreen2 = currentScreen + dirNorm * arrowTileSize;

    vec4 p = unproject(currentScreen2, currentProjected.z, currentProjected.w); 

    gl_Position = p;
}
